/* Last changed Time-stamp: <2007-10-29 17:34:19 htafer> */
/*                
	     Compute duplex structure of two RNA strands

			   c Ivo L Hofacker
			  Vienna RNA package
*/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>
#include <unistd.h>
#include <string.h>
#include <math.h>
#include <dirent.h>
#include "plex.h"
#include "ali_plex.h"
#include "fold.h"
#include "fold_vars.h"
#include "utils.h"
#include "stat.h"
#include "params.h"
#include "aln_util.h"
#include "PS_dot.h"
#include "pair_mat.h"
//---------------------for timer function
#include <sys/types.h>
#include <time.h>

clock_t BeginTimer()
{
  //timer declaration
  clock_t Begin; //initialize Begin
  Begin = clock() ; //start the timer
  return Begin;
}

clock_t EndTimer(clock_t begin)
{
  clock_t End;
  End = clock() ;   //stop the timer
  return End;
}


//--------------------end include timer
#define MIN2(A, B)      ((A) < (B) ? (A) : (B))
#define MAX2(A, B)      ((A) > (B) ? (A) : (B))
extern void read_parameter_file(const char fname[]);
extern int subopt_sorted;
//static int print_struc(duplexT const *dup);
static int ** average_accessibility_target(char **names, char **ALN, int number, char *access, double verhaeltnis,const int alignment_length, int binaries);
//static int ** average_accessibility_query(char **names, char **ALN, int number, char *access, double verhaeltnis);
static int get_max_u(const char *s, char delim);
static int ** read_plfold_i(char *fname, const int beg, const int end,double verhaeltnis, const int length);
//static int ** read_plfold_j(char *fname, const int beg, const int end,double verhaeltnis);
static int ** read_plfold_i_bin(char *fname, const int beg, const int end,double verhaeltnis, const int length);
static int get_sequence_length_from_alignment(char *sequence);
//take as argument a list of hit from an alignment interaction
//Accessibility can currently not be provided
static void aliprint_struct(FILE *Result, //result file
			    FILE *Target, //target alignment
			    FILE *Query //query alignment
			    );
static int linear_fit(int *a, int *b, int *c, int *d); //get linear fits;
/*@unused@*/
static int convert_plfold_i(char *fname);//convert test accessibility into bin accessibility.
static char rcsid[] = "$Id: rnaplex.c,v 1.10 2007/12/21 15:30:48 htafer Exp $";

static char  scale[] = "....,....1....,....2....,....3....,....4"
  "....,....5....,....6....,....7....,....8";

static void usage(void);
/*--------------------------------------------------------------------------*/

int main(int argc, char *argv[])
{
  
#define MAX_NUM_NAMES    500
  char *temp1[MAX_NUM_NAMES], *temp2[MAX_NUM_NAMES], *AS1[MAX_NUM_NAMES], *AS2[MAX_NUM_NAMES], *names1[MAX_NUM_NAMES], *names2[MAX_NUM_NAMES];
  
  char *s1=NULL, *s2=NULL, *line, *cstruc=NULL, *structure=NULL;
  char *line_q=NULL, *line_t=NULL;
  FILE *file1=NULL, *file2=NULL; // file containing the alignment;
  FILE *Result=NULL;   //file containing the results
  FILE *mRNA=NULL, *sRNA=NULL;
  char  fname[13];
  char  *ParamFile=NULL;
  //char  *Resultfile=NULL;
  int   fold_constrained=0;
  char  *ns_bases=NULL, *c;
  int   i, l, sym, r;
  double verhaeltnis=1;
  double deltaf;
  //double kT, sfact=1.07;
  int   pf=0, istty, delta=-INF;
  int noconv=0;
  int extension_cost=0;
  int deltaz=0;
  int alignment_length=40;
  int fast;
  int redraw=0;
  int binaries=0;
  int convert=0;
  char* tname=NULL;char *qname=NULL;
  fast=0;
  char *access=NULL;
  //int alignment;
  access=NULL;
  for (i=1; i<argc; i++) {
    if (argv[i][0]=='-') 
      switch ( argv[i][1] )
	{
	case 'q': /*file containing the sRNA*/
	  if(i==argc-1) usage();
	  qname=argv[++i];
	  break;
	case 't':/*file containing the mRNA*/
	  if(i==argc-1) usage();
	  tname=argv[++i];
	  break;
	case 's' : subopt_sorted=1;
	  break;	
	case 'l':  if (argv[i][2]!='\0') usage();
	  if(i==argc-1) usage();
	  r=sscanf(argv[++i], "%d", &alignment_length);
	  if (!r) usage();
	  break;
	case 'c':  if (argv[i][2]!='\0') usage();
	  if(i==argc-1) usage();
	  r=sscanf(argv[++i], "%d", &extension_cost);
	  if (!r) usage();
	  break;
	case 'z': if(argv[i][2]!='\0') usage();
	  if(i==argc-1) usage();
	  r=sscanf(argv[++i], "%d", &deltaz);
	  if (!r) usage();
	  break;
	case 'T':  if (argv[i][2]!='\0') usage();
	  if(i==argc-1) usage();
	  r=sscanf(argv[++i], "%lf", &temperature);
	  if (!r) usage();
	  break;
	case 'p':
	  fprintf(stderr, "partition function folding not yet implemented\n");
	  usage();
	  pf=1;
	  if (argv[i][2]!='\0')
	    (void) sscanf(argv[i]+2, "%d", &do_backtrack);
	  break;
	case 'n':
	  if ( strcmp(argv[i], "-noGU")==0) noGU=1;
	  if ( strcmp(argv[i], "-noCloseGU")==0) no_closingGU=1;
	  if ( strcmp(argv[i], "-noLP")==0) noLonelyPairs=1;
	  if ( strcmp(argv[i], "-nsp") ==0) {
	    if (i==argc-1) usage();
	    ns_bases = argv[++i];
	  }
	  if ( strcmp(argv[i], "-noconv")==0) noconv=1;
	  break;
	case 'e':
	  if (i>=argc-1) usage();
	  r=sscanf(argv[++i], "%lf", &deltaf);
	  if (r!=1) usage();
	  delta = (int) (0.1+deltaf*100);
	  break;
	case 'f':
	  if (i>=argc-1) usage();
	  r=sscanf(argv[++i], "%d", &fast);
	  if (r!=1) usage();
	  break;
	case 'a':
	  if (i==argc-1) usage();
	  access=argv[++i];
	  break;
	case 'd': dangles=0;
	  if (argv[i][2]!='\0') {
	    r=sscanf(argv[i]+2, "%d", &dangles);
	    if (r!=1) usage();
	  }
	  break;
	case 'P':
	  if (i==argc-1) usage();
	  ParamFile = argv[++i];
	  break;
	case 'I':
	  if (i==argc-1) usage();
	  Result = fopen(argv[++i],"r");
	  redraw=1;
	  if(Result == NULL){
	    fprintf(stderr, "can't open %s'n", argv[i]);
	    usage();
	  }
	  break;
	case 'V':
	  if(argv[i][2]!='\0') usage();
	  if(i==argc-1) usage();
	  r=sscanf(argv[++i], "%lf", &verhaeltnis);
	  if (!r) usage();
	  break;
	case 'C':
	  fold_constrained = 1; 
	  break;
	case 'b':
	  binaries=1;
	  break;
	case 'k':
	  convert=1;
	  break;
        case 'A':
          if(i==argc-2) usage();
	  file1 = fopen(argv[++i],"r");
          if (file1 == NULL) {
	    fprintf(stderr, "can't open %s\n", argv[i]);
	    usage();
      	  }
          file2 = fopen(argv[++i], "r");
          if (file2 == NULL) {
	    fprintf(stderr, "can't open %s\n", argv[i]);
	    usage();
      	  }
	  break;
	default: usage();
	  
	}
  }
  if(convert && access) {
    char pattern[8];
    strcpy(pattern,"_openen");
    DIR *dfd;
    struct dirent *dir;
    dfd=opendir(access);
    if(dfd){
      while((dir=readdir(dfd))!=NULL){
	int strPos=0;
	int PatternPos=0;
	char name[128];
	strcpy(name,access);
	strcat(name,"/");
	strcat(name,dir->d_name);
	while(name[strPos]!='\0'){
	  if(name[strPos]==pattern[0]){
	    while(name[strPos+PatternPos]==pattern[PatternPos] && pattern[PatternPos]!='\0' && name[strPos+PatternPos]!='\0'){
	      PatternPos++;
	    }
	    if(name[strPos+PatternPos]=='\0' && pattern[PatternPos]=='\0'){
	      convert_plfold_i(name);
	    }
	    else{
	      PatternPos=0;
	    }
	  }
	  strPos++;
	}
      }
    }
    closedir(dfd);
    return(0);
  }
  update_fold_params();
  if (ParamFile != NULL)
    read_parameter_file(ParamFile);
  if (ns_bases != NULL) {
    nonstandards = space(33);
    c=ns_bases;
    i=sym=0;
    if (*c=='-') {
      sym=1; c++;
    }
    while (*c!='\0') {
      if (*c!=',') {
	nonstandards[i++]=*c++; 
	nonstandards[i++]=*c;
	if ((sym)&&(*c!=*(c-1))) {
	  nonstandards[i++]=*c;
	  nonstandards[i++]=*(c-1);
	}
      }
      c++;
    }
  }
  int il_a,il_b,b_a,b_b;
  linear_fit(&il_a,&il_b,&b_a,&b_b);
  if((qname==NULL && tname) || (qname && tname==NULL)) usage();
  if(Result) {
    if(file1 == NULL || file2 == NULL){
      printf("Please give the name of the query and/or target file\n");
      printf("in the form of -A target_file query_file");
      usage();
    }
    else{
      printf("redraw %d",redraw);
      aliprint_struct(Result,file1,file2);
      fclose(Result);
      exit(0);
    }
  }
    
  if(((file1) &&  (!file2)) || ((!file1) &&  (file2))) {
    usage();
  }
  else if(qname && tname ) {
    if(!fold_constrained) {
      if(access) {
	char *id_s1=NULL;
	mRNA=fopen(tname, "r");
	if(mRNA==NULL){printf("%s: Wrong snoRNA file name\n", tname);return 0;}
	sRNA=fopen(qname, "r");
	if(sRNA==NULL){printf("%s: Wrong target file name\n", qname);return 0;}
	do {				/* main loop: continue until end of file */
	  if ((line_t = get_line(mRNA))==NULL) {
	    break;
	  }
	  /*parse line, get id for further accessibility fetching*/
	  while ((*line_t=='*')||(*line_t=='\0')||(*line_t=='>')) {
	    if(*line_t=='>'){
	      if(id_s1){
		free(id_s1);
		id_s1=NULL;
	      }
	      id_s1 = (char*) space (strlen(line_t)+2);
	      (void) sscanf(line_t,"%s",id_s1); 
	      memmove(id_s1, id_s1+1, strlen(id_s1));
	      free(line_t);
	      line_t=NULL;
	    }
	    if(line_t){
	      free(line_t);
	    }
	    if ((line_t = get_line(mRNA))==NULL) {
	      break;
	    }
	  } 
	  /*append N's to the sequence in order to avoid boundary checking*/
	  if ((line_t ==NULL) || (strcmp(line_t, "@") == 0)) {
	    if(id_s1){
	      free(id_s1);
	      id_s1=NULL;
	    }
	    break;
	  }
	  s1 = (char *) space(strlen(line_t)+1+20);
	  strcpy(s1,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
	  strcat(s1,line_t);
	  free(line_t);
	  strcat(s1,"NNNNNNNNNN\0");
	  int s1_len;
	  s1_len = strlen(s1);
	  for (l = 0; l < s1_len ; l++) {
	    s1[l] = toupper(s1[l]);
	    if (!noconv && s1[l] == 'T') s1[l] = 'U';
	  }
	  
	  /*read accessibility*/
	  int **access_s1;
	  char *file_s1;
	  file_s1 = (char *) space(sizeof(char) * (strlen(id_s1)+strlen(access)+20));
	  strcpy(file_s1, access); 
	  strcat(file_s1, "/");
	  strcat(file_s1, id_s1);
	  strcat(file_s1, "_openen");
	  if(!binaries){
	    access_s1 = read_plfold_i(file_s1,1,s1_len,verhaeltnis,alignment_length);
	  }
	  else{
	    strcat(file_s1,"_bin");
	    access_s1 = read_plfold_i_bin(file_s1,1,s1_len,verhaeltnis,alignment_length);
	  }
	  if(access_s1 == NULL){
	    printf("Accessibility file %s not found, look at next target RNA\n",file_s1);
	    free(file_s1);
	    free(s1);
	    free(id_s1);
	    id_s1=NULL;
	    continue;
	  }
	  do{
	    char *id_s2=NULL;
	    if ((line_q = get_line(sRNA))==NULL) {
	      break;
	    }
	    while ((*line_q=='*')||(*line_q=='\0')||(*line_q=='>')) {
	      if(*line_q=='>'){
		if(id_s2){
		  free(id_s2);
		  id_s2=NULL;
		}
		id_s2 = (char*) space (strlen(line_q)+2);
		(void) sscanf(line_q,"%s",id_s2); 
		memmove(id_s2, id_s2+1, strlen(id_s2));
		free(line_q);		
		line_q=NULL;
	      }
	      if(line_q){
		free(line_q);
	      }
	      if ((line_q = get_line(sRNA))==NULL) {
		break;
	      }
	    } 
	    if ((line_q ==NULL) || (strcmp(line_q, "@") == 0)){
	      if(id_s2){free(id_s2);id_s2=NULL;}
	      break;
	    }
	    if(!id_s2){
	      free(line_q);
	      continue;
	    }
	    s2 = (char *) space(strlen(line_q)+1+20);
	    strcpy(s2,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
	    strcat(s2,line_q);
	    free(line_q);
	    strcat(s2,"NNNNNNNNNN\0");
	    int s2_len;
	    s2_len = strlen(s2);
	    for (l = 0; l < s2_len ; l++) {
	      s2[l] = toupper(s2[l]);
	      if (!noconv && s2[l] == 'T') s2[l] = 'U';
	    }	  
	    int **access_s2;
	    char *file_s2;
	    file_s2 = (char *) space(sizeof(char) * (strlen(id_s2)+strlen(access)+20));
	    strcpy(file_s2, access); 
	    strcat(file_s2, "/");
	    strcat(file_s2, id_s2);
	    strcat(file_s2, "_openen");
	    if(!binaries){
	      access_s2 = read_plfold_i(file_s2,1,s2_len,verhaeltnis,alignment_length);
	    }
	    else{
	      strcat(file_s2,"_bin");
	      access_s2 = read_plfold_i_bin(file_s2,1,s2_len,verhaeltnis,alignment_length);
	    }
	    if(access_s2 == NULL){
	      printf("Accessibility file %s not found, look at next target RNA\n",file_s2);
	      free(file_s2);
	      free(s2);
	      free(id_s2);
	      id_s2=NULL;
	      continue;
	    }
	    printf(">%s\n>%s\n", id_s1, id_s2);
	    double begin = BeginTimer();
	    Lduplexfold_XS(s1,s2, (const int **) access_s1, (const int **) access_s2, delta,alignment_length, deltaz, fast, il_a, il_b, b_a, b_b);//, target_dead, query_dead);
	    float elapTicks;
	    float elapMilli, elapSeconds, elapMinutes;
	    elapTicks = (EndTimer(begin) - begin);
	    elapMilli = elapTicks/1000;
	    //	    printf("Millisecond %.2f\n",elapMilli);
	    free(id_s2);
	    free(file_s2);
	    free(s2);
	    id_s2=NULL;
	    i =  access_s2[0][0];           
	    while(--i>-1){
	      free(access_s2[i]);
	    }
	    free(access_s2);
	  }while(1);
	  free(id_s1);
	  id_s1=NULL;
	  free(file_s1);
	  free(s1);
	  rewind(sRNA);
	  i =  access_s1[0][0];
	  while(--i>-1){
	    free(access_s1[i]);
	  }
	  free(access_s1);
	}while(1);
	fclose(mRNA);
	fclose(sRNA);
      }
      else if(access==NULL){ //t and q are defined, but no accessibility is provided
	mRNA=fopen(tname, "r");
	if(mRNA==NULL){printf("%s: Wrong snoRNA file name\n", tname);return 0;}
	sRNA=fopen(qname, "r");
	if(sRNA==NULL){printf("%s: Wrong target file name\n", qname);return 0;}
	do {				/* main loop: continue until end of file */
	  char *id_s1=NULL; //header of the target file 
	  if ((line_t = get_line(mRNA))==NULL) {
	    break;
	  }
	  /*parse line, get id for further accessibility fetching*/
	  while ((*line_t=='*')||(*line_t=='\0')||(*line_t=='>')) {
	    if(*line_t=='>'){
	      if(id_s1){ //in case we have two header the one after the other
		free(id_s1);     //free the old header, a put the new one instead
		id_s1=NULL;
	      }
	      id_s1 = (char*) space (strlen(line_t)+2);
	      (void) sscanf(line_t,"%s",id_s1); 
	      memmove(id_s1, id_s1+1, strlen(id_s1));
	      free(line_t);		
	      line_t=NULL;
	    }
	    if(line_t){
	      free(line_t);
	    }
	    if ((line_t = get_line(mRNA))==NULL) {
	      break;
	    }
	  } 
	  /*append N's to the sequence in order to avoid boundary checking*/
	  if ((line_t ==NULL) || (strcmp(line_t, "@") == 0)) {
	    if(id_s1){
	      free(id_s1);
	      id_s1=NULL;
	    }
	    break;
	  }
	  s1 = (char *) space(strlen(line_t)+1+20);
	  strcpy(s1,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
	  strcat(s1,line_t);
	  free(line_t);
	  strcat(s1,"NNNNNNNNNN\0");
	  int s1_len;
	  s1_len = strlen(s1);
	  for (l = 0; l < s1_len ; l++) {
	    s1[l] = toupper(s1[l]);
	    if (!noconv && s1[l] == 'T') s1[l] = 'U';
	  }
	  do{
	    /*read sRNA files*/
	    char *id_s2=NULL;
	    if ((line_q = get_line(sRNA))==NULL) {
	      break;
	    }
	    while ((*line_q=='*')||(*line_q=='\0')||(*line_q=='>')) {
	      if(*line_q=='>') {
		if(id_s2){
		  free(id_s2);
		  id_s2=NULL;
		}
		id_s2 = (char*) space (strlen(line_q)+2);
		(void) sscanf(line_q,"%s",id_s2); 
		memmove(id_s2, id_s2+1, strlen(id_s2));
		free(line_q);		
		line_q=NULL;
	      }
	      if(line_q){
		free(line_q);
	      }
	      if ((line_q = get_line(sRNA))==NULL) {
		break;
	      }
	    } 
	    if ((line_q ==NULL) || (strcmp(line_q, "@") == 0)){
	      if(id_s2){free(id_s2);}
	      break;
	    }
	    if(!id_s2){
	      free(line_q);
	      continue;
	    }
	    s2 = (char *) space(strlen(line_q)+1+20);
	    strcpy(s2,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
	    strcat(s2,line_q);
	    free(line_q);
	    strcat(s2,"NNNNNNNNNN\0");
	    int s2_len;
	    s2_len = strlen(s2);
	    for (l = 0; l < s2_len ; l++) {
	      s2[l] = toupper(s2[l]);
	      if (!noconv && s2[l] == 'T') s2[l] = 'U';
	    }
	    printf(">%s\n>%s\n", id_s1, id_s2);
	    double begin = BeginTimer();
	    
	    Lduplexfold(s1,s2,delta,extension_cost,alignment_length, deltaz, fast,il_a, il_b, b_a, b_b);
	    float elapTicks;
	    float elapMilli, elapSeconds, elapMinutes;
	    elapTicks = (EndTimer(begin) - begin);
	    elapMilli = elapTicks/1000;
	    //	    printf("Millisecond %.2f\n",elapMilli);
	    printf("\n");
	    free(id_s2);
	    id_s2=NULL;
	    free(s2);
	  }while(1);
	  free(id_s1);
	  id_s1=NULL;
	  free(s1);
	  rewind(sRNA);
	}while(1);
	fclose(mRNA);
	fclose(sRNA);
      }
    }
    else{
      if(access) {
	char *id_s1=NULL;
	mRNA=fopen(tname, "r");
	if(mRNA==NULL){printf("%s: Wrong snoRNA file name\n", tname);return 0;}
	sRNA=fopen(qname, "r");
	if(sRNA==NULL){printf("%s: Wrong target file name\n", qname);return 0;}
	do {				/* main loop: continue until end of file */
	  if ((line_t = get_line(mRNA))==NULL) {
	    break;
	  }
	  /*parse line, get id for further accessibility fetching*/
	  while ((*line_t=='*')||(*line_t=='\0')||(*line_t=='>')) {
	    if(*line_t=='>'){
	      if(id_s1){ //in case we have two header the one after the other
		free(id_s1);     //free the old header, a put the new one instead
		id_s1=NULL;
	      }
	      id_s1 = (char*) space (strlen(line_t)+2);
	      (void) sscanf(line_t,"%s",id_s1); 
	      memmove(id_s1, id_s1+1, strlen(id_s1));
	      free(line_t);		
	      line_t=NULL;
	    }
	    if(line_t){
	      free(line_t);
	    }
	    if ((line_t = get_line(mRNA))==NULL) {
	      break;
	    }
	  } 
	  /*append N's to the sequence in order to avoid boundary checking*/
	  if ((line_t ==NULL) || (strcmp(line_t, "@") == 0)) {
	    if(id_s1){
	      free(id_s1);
	      id_s1=NULL;
	    }
	    break;
	  }
	  s1 = (char *) space(strlen(line_t)+1+20);
	  strcpy(s1,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
	  strcat(s1,line_t);
	  free(line_t);
	  strcat(s1,"NNNNNNNNNN\0");
	  int s1_len;
	  s1_len = strlen(s1);
	  for (l = 0; l < s1_len ; l++) {
	    s1[l] = toupper(s1[l]);
	    if (!noconv && s1[l] == 'T') s1[l] = 'U';
	  }
	  
	  /*read accessibility*/
	  int **access_s1;
	  char *file_s1;
	  file_s1 = (char *) space(sizeof(char) * (strlen(id_s1)+strlen(access)+20));
	  strcpy(file_s1, access); 
	  strcat(file_s1, "/");
	  strcat(file_s1, id_s1);
	  strcat(file_s1, "_openen");
	  if(!binaries){
	    access_s1 = read_plfold_i(file_s1,1,s1_len,verhaeltnis,alignment_length);
	  }
	  else{
	    strcat(file_s1,"_bin");
	    access_s1 = read_plfold_i_bin(file_s1,1,s1_len,verhaeltnis,alignment_length);
	  }
	  if(access_s1 == NULL){
	    printf("Accessibility file %s not found, look at next target RNA\n",file_s1);
	    free(file_s1);
	    free(s1);
	    free(id_s1);
	    id_s1=NULL;
	    continue;
	  }
	  do{
	    char *id_s2=NULL;
	    /*read sRNA files*/
	    if ((line_q = get_line(sRNA))==NULL) {
	      break;
	    }
	    while ((*line_q=='*')||(*line_q=='\0')||(*line_q=='>')) {
	      if(*line_q=='>'){
		if(id_s2){
		  free(id_s2);
		  id_s2=NULL;
		}
		id_s2 = (char*) space (strlen(line_q)+2);
		(void) sscanf(line_q,"%s",id_s2); 
		memmove(id_s2, id_s2+1, strlen(id_s2));
		free(line_q);		
		line_q=NULL;
	      }
	      if(line_q){
		free(line_q);
	      }
	      if ((line_q = get_line(sRNA))==NULL) {
		break;
	      }
	    } 
	    if ((line_q ==NULL) || (strcmp(line_q, "@") == 0)){
	      if(id_s2){free(id_s2);}
	      break;
	    }
	    //if ((line_t ==NULL) || (strcmp(line_t, "@") == 0)) break;
	    s2 = (char *) space(strlen(line_q)+1+20);
	    strcpy(s2,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
	    strcat(s2,line_q);
	    free(line_q);
	    strcat(s2,"NNNNNNNNNN\0");
	    int s2_len;
	    s2_len = strlen(s2);
	    for (l = 0; l < s2_len ; l++) {
	      s2[l] = toupper(s2[l]);
	      if (!noconv && s2[l] == 'T') s2[l] = 'U';
	    }
	    structure = (char *) space((unsigned) s2_len+1);
	    cstruc = get_line(sRNA);
	    if (cstruc!=NULL) {
	      int dn3=strlen(cstruc)-(s2_len-20);
	      strcpy(structure,"..........");
	      strncat(structure, cstruc, s2_len-20);
	      if(dn3>=0){
		strcat(structure,"..........\0");
	      }
	      else{
		while(dn3++){
		  strcat(structure,".");
		}
		strcat(structure,"\0");
	      }
	      free(cstruc);
	    }
	    else{
	      fprintf(stderr, "constraints missing\n");
	    }
	    int a = strchr(structure,'|') - structure;
	    int b = strrchr(structure,'|') - structure;
	    if(alignment_length < b-a+1){
	      nrerror("Maximal duplex length (-l option) is smaller than constraint on the structures\n. Please adjust the -l option accordingly\n");
	    }
	    int **access_s2;
	    char *file_s2;
	    file_s2 = (char *) space(sizeof(char) * (strlen(id_s2)+strlen(access)+20));
	    strcpy(file_s2, access); 
	    strcat(file_s2, "/");
	    strcat(file_s2, id_s2);
	    strcat(file_s2, "_openen");
	    if(!binaries){
	      access_s2 = read_plfold_i(file_s2,1,s2_len,verhaeltnis,alignment_length);
	    }
	    else{
	      strcat(file_s2,"_bin");
	      access_s2 = read_plfold_i_bin(file_s2,1,s2_len,verhaeltnis,alignment_length);
	    }
	    if(access_s2 == NULL){
	      printf("Accessibility file %s not found, look at next target RNA\n",file_s2);
	      free(file_s2);free(s2);free(id_s2);
	      continue;
	    }
	    printf(">%s\n>%s\n", id_s1, id_s2);
	    Lduplexfold_CXS(s1,s2, (const int **) access_s1, (const int **) access_s2, delta,alignment_length, deltaz, fast,structure,il_a, il_b, b_a, b_b);//, target_dead, query_dead);
	    free(id_s2);
	    free(file_s2);
	    free(s2);
	    i =  access_s2[0][0];           
	    while(--i>-1){
	      free(access_s2[i]);
	    }
	    free(access_s2);free(structure);
	  }while(1);
	  free(id_s1);
	  id_s1=NULL;
	  free(file_s1);
	  free(s1);
	  rewind(sRNA);
	  i =  access_s1[0][0];
	  while(--i>-1){
	    free(access_s1[i]);
	  }
	  free(access_s1);
	}while(1);
	fclose(mRNA);
	fclose(sRNA);
      }
      else if(access==NULL){ //t and q are defined, but no accessibility is provided
	char *id_s1=NULL;
	mRNA=fopen(tname, "r");
	if(mRNA==NULL){printf("%s: Wrong snoRNA file name\n", tname);return 0;}
	sRNA=fopen(qname, "r");
	if(sRNA==NULL){printf("%s: Wrong target file name\n", qname);return 0;}
	do {				/* main loop: continue until end of file */
	  if ((line_t = get_line(mRNA))==NULL) {
	    break;
	  }
	  /*parse line, get id for further accessibility fetching*/
	  while ((*line_t=='*')||(*line_t=='\0')||(*line_t=='>')) {
	    if(*line_t=='>'){
	      if(id_s1){ //in case we have two header the one after the other
		free(id_s1);     //free the old header, a put the new one instead
		id_s1=NULL;
	      }
	      id_s1 = (char*) space (strlen(line_t)+2);
	      (void) sscanf(line_t,"%s",id_s1); 
	      memmove(id_s1, id_s1+1, strlen(id_s1));
	      free(line_t);		
	      line_t=NULL;
	    }
	    if(line_t){
	      free(line_t);
	    }
	    if ((line_t = get_line(mRNA))==NULL) {
	      break;
	    }
	  } 
	  /*append N's to the sequence in order to avoid boundary checking*/
	  //if ((line_t ==NULL) || (strcmp(line_t, "@") == 0)) break;
	  if ((line_t ==NULL) || (strcmp(line_t, "@") == 0)) {
	    if(id_s1){
	      free(id_s1);
	      id_s1=NULL;
	    }
	    break;
	  }
	  s1 = (char *) space(strlen(line_t)+1+20);
	  strcpy(s1,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
	  strcat(s1,line_t);
	  free(line_t);
	  strcat(s1,"NNNNNNNNNN\0");
	  int s1_len;
	  s1_len = strlen(s1);
	  for (l = 0; l < s1_len ; l++) {
	    s1[l] = toupper(s1[l]);
	    if (!noconv && s1[l] == 'T') s1[l] = 'U';
	  }
	  do{
	    char *id_s2=NULL;
	    /*read sRNA files*/
	    if ((line_q = get_line(sRNA))==NULL) {
	      break;
	    }
	    while ((*line_q=='*')||(*line_q=='\0')||(*line_q=='>')) {
	      if(*line_q=='>'){
		if(id_s2){
		  free(id_s2);
		  id_s2=NULL;
		}
		id_s2 = (char*) space (strlen(line_q)+2);
		(void) sscanf(line_q,"%s",id_s2); 
		memmove(id_s2, id_s2+1, strlen(id_s2));
		free(line_q);		
		line_q=NULL;
	      }
	      if(line_q){
		free(line_q);
	      }
	      if ((line_q = get_line(sRNA))==NULL) {
		break;
	      }
	    } 
	    //if ((line_t ==NULL) || (strcmp(line_t, "@") == 0)) break;
	    if ((line_q ==NULL) || (strcmp(line_q, "@") == 0)){
	      if(id_s2){free(id_s2);}
	      break;
	    }
	    s2 = (char *) space(strlen(line_q)+1+20);
	    strcpy(s2,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
	    strcat(s2,line_q);
	    free(line_q);
	    strcat(s2,"NNNNNNNNNN\0");
	    int s2_len;
	    s2_len = strlen(s2);
	    for (l = 0; l < s2_len ; l++) {
	      s2[l] = toupper(s2[l]);
	      if (!noconv && s2[l] == 'T') s2[l] = 'U';
	    }
	    structure = (char *) space((unsigned) s2_len+1);
	    cstruc = get_line(sRNA);
	    if (cstruc!=NULL) {
	      int dn3=strlen(cstruc)-(s2_len-20);
	      strcpy(structure,"..........");
	      strncat(structure, cstruc, s2_len-20);
	      if(dn3>=0){
		strcat(structure,"..........\0");
	      }
	      else{
		while(dn3++){
		  strcat(structure,".");
		}
		strcat(structure,"\0");
	      }
	      free(cstruc);
	    }
	    else{
	      fprintf(stderr, "constraints missing\n");
	    }
	    int a = strchr(structure,'|') - structure;
	    int b = strrchr(structure,'|') - structure;
	    if(alignment_length < b-a+1){
	      nrerror("Maximal duplex length (-l option) is smaller than constraint on the structures\n. Please adjust the -l option accordingly\n");
	    }
	    printf(">%s\n>%s\n", id_s1, id_s2);
	    double begin = BeginTimer();
	    Lduplexfold_C(s1,s2,delta,extension_cost,alignment_length, deltaz, fast,structure,il_a,il_b,b_a, b_b);
	    float elapTicks;
	    float elapMilli, elapSeconds, elapMinutes;
	    elapTicks = EndTimer(begin);
	    elapMilli = elapTicks/1000;
	    //	    printf("Millisecond %.2f\n",elapMilli);
	    free(id_s2);free(s2);free(structure);
	  }while(1);
	  free(id_s1);
	  id_s1=NULL;
	  free(s1);
	  rewind(sRNA);
	}while(1);
	fclose(mRNA);
	fclose(sRNA);
      }
    }
  }
  else if(!file1 && !file2) {
    istty = isatty(fileno(stdout))&&isatty(fileno(stdin));
    char *id_s1=NULL, *id_s2=NULL;	
    if ((fold_constrained)  && (istty)) {
      printf("Input constraints using the following notation:\n");
      printf("| : paired with another base\n");
      printf(". : no constraint at all\n");
      printf("x : base must not pair\n");
    }
    do {				/* main loop: continue until end of file */
      //duplexT mfe, *subopt	
      if (istty) {
	printf("\nInput two sequences (one line each); @ to quit\n");
	printf("%s\n", scale);
      }
      fname[0]='\0';
      
      if ((line = get_line(stdin))==NULL) break;
      /* skip empty lines, comment lines, name lines */
      while ((*line=='*')||(*line=='\0')||(*line=='>')) {
	printf("%s\n", line); 
	if(*line=='>'){
	  id_s1 = (char*) space (strlen(line)+2);
	  (void) sscanf(line,"%s",id_s1); 
	  memmove(id_s1, id_s1+1, strlen(id_s1));
	}
	free(line);		
	if ((line = get_line(stdin))==NULL) {
	  free(id_s1);
	  break;
	}
      } 
      if ((line ==NULL) || (strcmp(line, "@") == 0)) break;
      
      s1 = (char *) space(strlen(line)+1+20);
      strcpy(s1,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
      strcat(s1,line);
      free(line);
      strcat(s1,"NNNNNNNNNN\0");
      if ((line = get_line(stdin))==NULL) break;
      /* skip comment lines and get filenames */
      
      while ((*line=='*')||(*line=='\0')||(*line=='>')) {
	printf("%s\n", line); 
	if(*line=='>'){
	  id_s2 = (char*) space (strlen(line)+2);
	  (void) sscanf(line,"%s",id_s2); 
	  memmove(id_s2, id_s2+1, strlen(id_s2));
	}
	free(line);		
	if ((line = get_line(stdin))==NULL) {
	  free(id_s2);break;
	}
      } 
      if ((line ==NULL) || (strcmp(line, "@") == 0)) break;
      
      s2 = (char *) space(strlen(line)+1+20);
      strcpy(s2,"NNNNNNNNNN"); /*add NNNNNNNNNN to avoid boundary check*/
      strcat(s2,line);
      free(line);
      strcat(s2,"NNNNNNNNNN\0");
      
      int n1=strlen(s1);
      int n2=strlen(s2);

      
      structure = (char *) space((unsigned) n2+1);
      if (fold_constrained) {
	cstruc = get_line(stdin);
	if (cstruc!=NULL){
	  int dn3=strlen(cstruc)-(n2-20);
	  strcpy(structure,"..........");
	  strncat(structure, cstruc, n2-20);
	  if(dn3>=0){
	    strcat(structure,"..........\0");
	  }
	  else{
	    while(dn3++){
	      strcat(structure,".");
	    }
	    strcat(structure,"\0");
	  }

	  free(cstruc);
	}
	else
	  fprintf(stderr, "constraints missing\n");
      }
      for (l = 0; l < n1 ; l++) {
	s1[l] = toupper(s1[l]);
	if (!noconv && s1[l] == 'T') s1[l] = 'U';
      }
      for (l = 0; l < n2 ; l++) {
	s2[l] = toupper(s2[l]);
	if (!noconv && s2[l] == 'T') s2[l] = 'U';
      }
      if (istty)
	printf("lengths = %d,%d\n", strlen(s1), strlen(s2));

      if(alignment_length==0){alignment_length=40;}

      if(access==NULL){
	if(!fold_constrained){
	  Lduplexfold(s1,s2,delta,extension_cost,alignment_length, deltaz, fast,il_a,il_b,b_a,b_b);
	}
	else{
	  int a = strchr(structure,'|') - structure;
	  int b = strrchr(structure,'|') - structure;
	  if(alignment_length < b-a+1){
	    nrerror("Maximal duplex length (-l option) is smaller than constraint on the structures\n. Please adjust the -l option accordingly\n");
	  }
	  Lduplexfold_C(s1,s2,delta,extension_cost,alignment_length, deltaz,fast,structure,il_a,il_b,b_a,b_b);
	}
      }
      else{
	int **access_s1, **access_s2;
	char *file_s1, *file_s2;
	int s1_len, s2_len;
	s1_len = strlen(s1);
	s2_len = strlen(s2);
	file_s1 = (char *) space(sizeof(char) * (strlen(id_s1)+strlen(access)+20));
	file_s2 = (char *) space(sizeof(char) * (strlen(id_s2)+strlen(access)+20));
	strcpy(file_s1, access); strcpy(file_s2, access); 
	strcat(file_s1, "/"); strcat(file_s2, "/"); 
	strcat(file_s1, id_s1);strcat(file_s2, id_s2);
	strcat(file_s1, "_openen");strcat(file_s2, "_openen");
	if(!binaries){
	  access_s1 = read_plfold_i(file_s1,1,s1_len,verhaeltnis,alignment_length);
	}
	else{
	  strcat(file_s1,"_bin");
	  access_s1 = read_plfold_i_bin(file_s1,1,s1_len,verhaeltnis,alignment_length);
	}
	if(access_s1 == NULL){
	  free(file_s1);
	  free(s1);
	  free(s2);
	  free(file_s2);
	  free(id_s1);
	  free(id_s2);
	  continue;
	}
	if(!binaries){
	  access_s2 = read_plfold_i(file_s2,1,s2_len,verhaeltnis,alignment_length);
	}
	else{
	  strcat(file_s2,"_bin");
	  access_s2 = read_plfold_i_bin(file_s2,1,s2_len,verhaeltnis,alignment_length);
	}
	if(access_s2 == NULL){
	  free(access_s1);
	  free(file_s1);
	  free(s1);
	  free(s2);
	  free(file_s2);
	  free(id_s1);
	  free(id_s2);     
	  continue;
	}
	if(!fold_constrained){
	  Lduplexfold_XS(s1,s2, (const int **) access_s1, (const int **) access_s2, delta,alignment_length, deltaz, fast,il_a,il_b,b_a,b_b);//, target_dead, query_dead);
	  
	}
	else{
	  int a = strchr(structure,'|') - structure;
	  int b = strrchr(structure,'|') - structure;
	  if(alignment_length < b-a+1){
	    nrerror("Maximal duplex length (-l option) is smaller than constraint on the structures\n. Please adjust the -l option accordingly\n");
	  }
	  Lduplexfold_CXS(s1,s2, (const int **) access_s1, (const int **) access_s2, delta,alignment_length, deltaz, fast,structure,il_a,il_b,b_a,b_b);//, target_dead, query_dead);
	}
	i =  access_s1[0][0];           
	while(--i>-1){
	  free(access_s1[i]);
	}
	i =  access_s2[0][0];           
	while(--i>-1){
	  free(access_s2[i]);
	}
	free(access_s1);
	free(access_s2);
	free(file_s1);
	free(file_s2);
	free(id_s1);id_s1=NULL;
	free(id_s2);id_s2=NULL;
      }
      free(s1); free(s2);s1=NULL;s2=NULL;
      free(structure);
      printf("\n");
      if(id_s1){free(id_s1);}if(id_s2){free(id_s2);}
    } while (1);
    if(s1){
      free(s1);
    }
    if(s2){
      free(s2);
    }
    //if(id_s1){free(id_s1);}if(id_s2){free(id_s2);}
  }
  else{
    int n_seq, n_seq2;	
    n_seq = read_clustal(file1, temp1, names1);
    n_seq2 = read_clustal(file2, temp2, names2);
    fclose(file1); fclose(file2);
    if (n_seq != n_seq2){
      for (i=0; temp1[i]; i++) {
	free(temp1[i]); 
	free(temp2[i]); 
      }  
      nrerror("unequal number of seqs in alignments");
    }
    for(i=0;temp1[i];i++){
      AS1[i] = (char*) space((strlen(temp1[i])+21)*sizeof(char));
      AS2[i] = (char*) space((strlen(temp2[i])+21)*sizeof(char));
      strcpy(AS1[i],"NNNNNNNNNN");
      strcpy(AS2[i],"NNNNNNNNNN");
      strcat(AS1[i],temp1[i]);
      strcat(AS2[i],temp2[i]);
      strcat(AS1[i],"NNNNNNNNNN\0");
      strcat(AS2[i],"NNNNNNNNNN\0");
    }
    for (i=0; temp1[i]; i++) {
      free(temp1[i]); 
      free(temp2[i]); 
    }  
    AS1[n_seq]=NULL;
    AS2[n_seq]=NULL;


    if(access==NULL){
      aliLduplexfold((const char**) AS1,(const char**) AS2, n_seq*delta,  extension_cost, alignment_length,  deltaz, fast,il_a,il_b,b_a,b_b);
    }
    else{
      int** target_access=NULL, **query_access=NULL;
      target_access=average_accessibility_target(names1, AS1, n_seq, access,verhaeltnis,alignment_length,binaries); //get averaged accessibility for alignments
      query_access =average_accessibility_target(names2, AS2, n_seq, access,verhaeltnis,alignment_length,binaries);
      if(!(target_access && query_access)){
	for (i=0; AS1[i]; i++) {
	  free(AS1[i]); free(names1[i]);
	  free(AS2[i]); free(names2[i]);
	}
	return 0;
      }
      aliLduplexfold_XS((const char**)AS1, (const char**)AS2, (const int **) target_access, (const int **) query_access, n_seq*delta, alignment_length,  deltaz, fast, il_a, il_b, b_a,b_b);	
      if(!(target_access==NULL)){                     		
	i = target_access[0][0];               	
	while(--i>-1) {
	  free(target_access[i]);
	}
	free(target_access);
      }
      if(!(query_access==NULL)){
	i = query_access[0][0];               	
	while(--i>-1) {
	  free(query_access[i]);
	}
	free(query_access);
      } 
    }
    for (i=0; AS1[i]; i++) {
      free(AS1[i]); free(names1[i]);
      free(AS2[i]); free(names2[i]);
    }  
  }
  return 0;
}

static void usage(void)
{
  nrerror("usage:\n"
	  "RNAplex [-e energy threshold] [-z search range] [-f short output [0|1]] [-c extension_cost] [-l maximal length of an alignment][-a accessibility directory]\n");
}

//static int print_struc(duplexT const *dup) {
//  /*this function print out the structure of a hybridization site
//    and return the value from which one can begin to look for the next
//    non-overlappig hybrid*/
//  
//  int l1;
//  float energy=dup->energy*0.01;
//  int init=dup->end;
//  l1 = strchr(dup->structure, '&')-dup->structure;
//  printf("%s %3d,%-3d : %3d,%-3d (%5.2f)\n", dup->structure, init-l1-1,
//	 init-2, dup->j+l1+2-strlen(dup->structure), dup->j, energy);
//  return init-l1-1;
//}
static int ** read_plfold_i(char *fname, const int beg, const int end, double verhaeltnis, const int length)
{
  double begin = BeginTimer();
  FILE *in=fopen(fname,"r");
  if(in==NULL){
    fprintf(stderr,"File ' %s ' open error\n",fname);
    return NULL;
  }
  int i,j;
  int **access;
  int offset,temp;
  temp=0;offset=0;
  int seq_pos;
  int beg_r, end_r;
  beg_r=beg;
  end_r=end;
  char tmp[2048]={0x0};
  //char *ptr;
  if(fgets(tmp,sizeof(tmp),in)==0){
    perror("Empty File");
  }
  if(strchr(tmp,'>')){
    fprintf(stderr,"file %s is not in RNAplfold format\n",fname);
    return NULL;
  }
  if(fgets(tmp,sizeof(tmp),in)==0){
    fprintf(stderr,"No accessibility data\n");
    return NULL;
  }
  int dim_x;
  dim_x=get_max_u(tmp,'\t');
  if(length>dim_x){
    printf("Interaction length %d is larger than the length of the largest region %d \nfor which the opening energy was computed (-u parameter of RNAplfold)\n",length,dim_x);  
    printf("Please recompute your profiles with a larger -u or set -l to a smaller interaction length\n");
    return NULL;
  }
  access = (int**) space(sizeof(int *) * (dim_x+2));
  for(i=0; i< dim_x+2; i++){
    access[i] =(int *) space(sizeof(int) * (end_r-beg_r+1)); //normally +1
  }
  for(i=0;i<end_r -  beg_r +1;i++){
    for(j=0;j<dim_x+2;j++){
      access[j][i]=INF;
    }
  }
  access[0][0]=dim_x+2;
  while(fgets(tmp,sizeof(tmp),in)!=0 && --end_r > 10) /* read a record, before we had --end_r > 10*/
    {
      float n;
      //int i;
      //int u;
      beg_r--;
      if(beg_r<1){
	if(sscanf(tmp,"%d\t%n",&seq_pos,&temp)==1){
	  offset+=temp;
	  int count;
	  count=1;
	  while(sscanf(tmp+offset,"%f%n",&n,&temp)==1 ){
	    offset+=temp;
	    //seq_pos - beg allows to get the accessibility right
	    access[count][seq_pos - beg +11]=(int)  rint( 100 * n); //round the number
	    access[count][seq_pos - beg +11]*=verhaeltnis; //10 stands here for the number of nucleotides
	    count++;
	  }
	  offset=0;
	}
      }
      
    }
  fclose(in);
  float elapTicks;
  float elapMilli, elapSeconds, elapMinutes;
  elapTicks = (EndTimer(begin) - begin);
  elapMilli = elapTicks/1000;
  //printf("read_plfold_i Millisecond %.2f\n",elapMilli);
  return access;
}

static int convert_plfold_i(char *fname)
{
  int i,k;
  FILE *in=fopen(fname,"r");
  if(in==NULL){
    fprintf(stderr,"File ' %s ' open error\n",fname);
    return -1;
  }
  char tmp[2048]={0x0};
  if(fgets(tmp,sizeof(tmp),in)==0){
    perror("Empty File");
  }
  if(strchr(tmp,'>')){
    fprintf(stderr,"file %s is not in RNAplfold format\n",fname);
    return -1;
  }
  if(fgets(tmp,sizeof(tmp),in)==0){
    fprintf(stderr,"No accessibility data\n");
    return -1;
  }
  int u_length;
  u_length=get_max_u(tmp,'\t'); //get the x dimension
  char c;
  int length=0;
  while((c=fgetc(in))!=EOF){
    if(c=='\n'){
      length++;
    }
  }
  fclose(in);
  int **access = read_plfold_i(fname,1,length+20,1,u_length);
  char *outname;
  outname = (char*) space((strlen(fname)+5)*sizeof(char));
  strcpy(outname,fname);
  strcat(outname,"_bin");
  FILE *fp=fopen(outname,"wb");
  int p[1];
  p[0]=1000000;
  int count;
  for (i=0; i< u_length +2 ; i++) {
    fwrite(access[i],sizeof(int),length+20,fp);
    free(access[i]);
  }
  fseek(fp,0,SEEK_SET);
  fwrite(&u_length,sizeof(int),1,fp);
  fwrite(&length,sizeof(int),1,fp);
  free(outname);
  fclose(fp);
  free(access);
  return 1;
}
 
//  if(length>dim_x){
//    printf("Interaction length %d is larger than the length of the largest region %d \nfor which the opening energy was computed (-u parameter of RNAplfold)\n",length,dim_x);  
//    printf("Please recompute your profiles with a larger -u or set -l to a smaller interaction length\n");
//    return NULL;
//  }
//  access = (int**) space(sizeof(int *) * (dim_x+2));
//  for(i=0; i< dim_x+2; i++){
//    access[i] =(int *) space(sizeof(int) * (end_r-beg_r+1)); //normally +1
//  }
//  for(i=0;i<end_r -  beg_r +1;i++){
//    for(j=0;j<dim_x+2;j++){
//      access[j][i]=INF;
//    }
//  }
//  access[0][0]=dim_x+2;
//  while(fgets(tmp,sizeof(tmp),in)!=0 && --end_r > 10) /* read a record, before we had --end_r > 10*/
//    {
//      float n;
//      //int i;
//      //int u;
//      beg_r--;
//      if(beg_r<1){
//	if(sscanf(tmp,"%d\t%n",&seq_pos,&temp)==1){
//	  offset+=temp;
//	  int count;
//	  count=1;
//	  while(sscanf(tmp+offset,"%f%n",&n,&temp)==1 ){
//	    offset+=temp;
//	    //seq_pos - beg allows to get the accessibility right
//	    access[count][seq_pos - beg +11]=(int)  rint( 100 * n); //round the number
//	    access[count][seq_pos - beg +11]*=verhaeltnis; //10 stands here for the number of nucleotides
//	    count++;
//	  }
//	  offset=0;
//	}
//      }
//      
//    }
//  fclose(in);
//  float elapTicks;
//  float elapMilli, elapSeconds, elapMinutes;
//  elapTicks = (EndTimer(begin) - begin);
//  elapMilli = elapTicks/1000;
//  printf("read_plfold_i Millisecond %.2f\n",elapMilli);
//  return access;
//}






static int ** read_plfold_i_bin(char *fname, const int beg, const int end, double verhaeltnis, const int length)
{
  double begin = BeginTimer();
  FILE *fp=fopen(fname,"rb");
  int seqlength;
  if(fp==NULL){
    fprintf(stderr,"File ' %s ' open error\n",fname);
    return NULL;
  }
  int *first_line;
  first_line =(int *) space(sizeof(int) * (end - beg+1)); //check length of the line LOOK at read_plfold_i
  fread(first_line,sizeof(int),(end-beg)+1,fp);                  //get the value of the u option
  int lim_x;                                                
  lim_x=first_line[0];
  seqlength=first_line[1];                                  //length of the sequence RNAplfold was ran on.
  if(length > lim_x){
    printf("Interaction length %d is larger than the length of the largest region %d \nfor which the opening energy was computed (-u parameter of RNAplfold)\n",length,lim_x);  
    printf("Please recompute your profiles with a larger -u or set -l to a smaller interaction length\n");
    return NULL;
  }        
  fseek(fp,0,SEEK_SET);                                     //set pointer to the begining of the file
  int **access;                                             //here we store the access  values
  access = (int**) space(sizeof(int *) * (lim_x+2));          
  int count;
  long int position;
  for(count=0; count < lim_x+2; count++){                     //read file
    access[count] = (int*) space(sizeof(int) * (end - beg+1)); //declare array length
    //now we should be sure to read the right position
    //we first need a seek to the right position
    //0->9 line begin, + begin 
    //here we need information about the total line length..., we can get this if this is saved by RNAplfold...
    position=ftell(fp);
    fseek(fp,(beg-1)*sizeof(int),SEEK_CUR); //go to the desired position, note the 10 offset
    position=ftell(fp);
    fread(access[count],sizeof(int),(end-beg)+1,fp); //read the needed number of accessibility values
    position=ftell(fp);
    fseek(fp,(seqlength-end+20)*sizeof(int),SEEK_CUR); //place to the begining of the next file
    position=ftell(fp);
  }
  fseek(fp,0,SEEK_SET);
  access[0][0]=lim_x;
  access[0][0]+=2;
  fclose(fp); //close file
  float elapTicks;
  float elapMilli, elapSeconds, elapMinutes;
  free(first_line);
  elapTicks = (EndTimer(begin) - begin);
  elapMilli = elapTicks/1000;
  //printf("read_plfold_i_bin Millisecond %.2f\n",elapMilli); //return time
  return access; //return access
}







static int get_max_u(const char *s, char delim){
  char * pch;
  int total;
  total=0;
  pch = strchr(s, delim);
  total++;
  while(pch!=NULL){
    pch=strchr(pch+1, delim);
    total++;
  }
  return total-2;
}


static int **average_accessibility_target(char **names, char **ALN, int number, char *access,double verhaeltnis,const int alignment_length,int binaries)
{
  int i;
  int *** master_access = NULL; //contains the accessibility arrays for different
  int ** average_access = NULL; //contains the averaged accessibility
  int aln_size=strlen(ALN[0]); //aln size -> define size of the averaged accessibility array
  int * index; //contains the index used for navigating inside the alignments
  long long int begin, end; //contains the begin and end region to read the accessibility
  index = (int*) space(sizeof(int) * number);
  for(i=0; i<number; i++){
    index[i]=1;
  }
  master_access = (int ***) space(sizeof(int**) * number); 
  int dim_x; //contains the minimal of the maximum u length for all sequences of the alignment
  dim_x=INF;
  int u;
  int location_flag; //location flag checks if all line contains a location flag or not
  //if some line contains location information and other not, return a warning but keep going;
  //The location flag will be set to TRUE if the following pattern is found
  // sequence-name/[0-9]+-[0-9]+ 
  // |----NAME----||BEG|  |END|
  char bla[255];
  if(sscanf(names[0],"%255[^/]/%lld-%lld",bla, &begin,&end)==3){ //initialize location_flag;
    location_flag=1;
  }   
  else{
    location_flag=0;
  }
  char *file_s1=NULL;
  for(i=0; i< number; i++) {    // be careful!!!! Name should contain all characters from begin till the "/" character
    //char *s1;
    file_s1 = (char *) space(sizeof(char) * (strlen(names[i])+strlen(access)+20));
    begin=1;
    int sequence_length = get_sequence_length_from_alignment(ALN[i]);
    end=sequence_length;
    if(sscanf(names[i],"%255[^/]/%lld-%lld",bla, &begin,&end)==3){
      //if subsequence and range do not have the same length, stop RNAplex.
      if(end-begin+1 != sequence_length-20){ 
	printf("Your range %lld %lld in sequence %s does not correspond to the sequence length %d\n",begin,end,names[i],sequence_length-20);
	printf("Please check your input alignments and rerun RNAplex");
	int a=0;
	if(master_access != NULL){
	  for(a=0; a<i; a++){
	    int j;
	    j = master_access[a][0][0];
	    while(--j>-1){
	      free(master_access[a][j]);
	  }
	    free(master_access[a]);	
	  }
	}
	free(master_access);
	free(index);free(file_s1);
	return average_access;
      }
      
      end+=20; //add 20 to the end, in order to take the N's into account
      if(location_flag==0){
	fprintf(stderr,"\n!! Line %d in your target alignment contains location information\n",i+1);
	fprintf(stderr,"while line %d did not. PLEASE CHECK your alignments!!\n",i);
	fprintf(stderr,"RNAplex will continue the target search.\n");
      }
      location_flag=1;
      strcpy(file_s1, access);
      strcat(file_s1, "/");
      strcat(file_s1,bla);
    }
    else{
      if(location_flag==1){
	fprintf(stderr,"\n!! Line %d in your target alignment does not contain location information\n",i+1);
	fprintf(stderr,"while line %d in your target alignment did. PLEASE CHECK your alignments!!\n",i);
	fprintf(stderr,"RNAplex will continue the target search.\n");
      }
      location_flag=0;
      strcpy(file_s1, access);
      strcat(file_s1, "/");
      strcat(file_s1,names[i]);
    }
    strcat(file_s1, "_openen");
    if(!binaries){
      master_access[i]=read_plfold_i(file_s1,begin,end,verhaeltnis,alignment_length); //read
    }
    else{
      strcat(file_s1,"_bin");
      master_access[i]=read_plfold_i_bin(file_s1,begin,end,verhaeltnis,alignment_length); //read
    }
    
    free(file_s1);
    file_s1=NULL;
    dim_x=MIN2(dim_x, master_access[i][0][0]);
  }
  average_access = (int **) space(sizeof(int*) * (dim_x));
  for(i=0;i<dim_x;i++){
    average_access[i] = (int *) space(sizeof(int) * (aln_size+9));  //average_access is of the length of the alignment
    //while master_access is of the length of the sequences.
  }
  average_access[0][0]=dim_x;
  for(i=1;i<aln_size;i++){ //go through all aln position
    int j;
    int count_j=number;
    for(j=0;j<number; j++){ //go through all aln members
      if(!(ALN[j][i-1]=='-')) { //if we have a gap, skip this position
	for(u=1; u<dim_x; u++){ //go through all u
	  if(index[j]<u+10){continue;}
	  average_access[u][i]+=master_access[j][u][index[j]]; //index[j] position in sequence j
	}
	index[j]++; //increase position in sequence
      }
      else{
	count_j--;
      }
    }
    if(!(count_j>0)){printf("Alignment contains only gap at column %d\n exiting program\n", i); return NULL;}
    for(u=1; u<dim_x; u++){
      average_access[u][i]=(int) rint(average_access[u][i]/(count_j));      
    }
  }
  //free(index);
  for(i=0; i<number; i++){
    int j;
    j =  master_access[i][0][0];
    while(--j>-1){
      free(master_access[i][j]);
    }
    free(master_access[i]);	
  }
  free(master_access);
  free(index);	
  return average_access;
}

static void aliprint_struct(FILE *Result, //result file
			    FILE *mrna, //target alignment
			    FILE *query //query alignment
			    
			    ){
  char *result=NULL;
  //char *line=NULL;
  char *AS1[MAX_NUM_NAMES], *AS2[MAX_NUM_NAMES], *names1[MAX_NUM_NAMES], *names2[MAX_NUM_NAMES]; //define arrays for names and sequences 
  int n_seq, n_seq2,i,s;


  //declare arrays, read sequence file and parse alignments
  n_seq =  read_clustal(mrna, AS1, names1);
  n_seq2 = read_clustal(query, AS2, names2);
  if (n_seq != n_seq2){
    for (i=0; AS1[i]; i++) {
      free(AS1[i]); 
      free(AS2[i]); 
    }  
    nrerror("unequal number of seqs in alignments");
  }
  AS1[n_seq]=NULL;
  AS2[n_seq]=NULL;
  //read result file
  do {				/* main loop: continue until end of file */
    if ((result = get_line(Result))==NULL) {
      free(result);
      break;
    }
    if(strchr(result, '(') && strchr(result,'&') && strchr(result, '(')&& strchr(result, ',') && strchr(result,':') && strchr(result, '-')){
      char *structure,*pos;
      int tbegin,tend,qbegin,qend; //store coordinates for target and query
      //parse results
      ////parse structure
      int length,posi;
      pos = strchr(result, ' ');
      posi = (int)  (pos - result);
      length = posi;
      structure = (char *) space((length+1) * sizeof(char));
      sscanf(result,"%s",structure);
      //parse coordinates
      //important for slicing the alignments.
      
      sscanf(pos, "%10d,%10d", &tbegin,&tend);

      pos = strchr(pos, ':');
      pos = strchr(pos, ' '); 
      sscanf(pos, "%10d,%10d", &qbegin,&qend);
      int l1 = strchr(structure, '&')-structure;
      
      //now slice the alignment
      char **target;
      target = (char**) space((n_seq+1)*sizeof(char*));
      for(s=0;s<n_seq;s++){
	target[s] = (char *) space((length+1)*sizeof(char));
	strncpy(target[s], (AS1[s]+tbegin-1), l1);
	strcat(target[s],"&");
	strncat(target[s], (AS2[s]+qbegin-1), length-l1-1);
	target[s][length]='\0';
      }
      target[n_seq]=NULL;
      //now produce output name
      
      int length_name = strlen(names1[0]) + strlen(names2[0])+1; //take first 2 name in alignment
      char *psoutput = (char*) space((length_name + 100)*sizeof(char));
      char str[16];
      
      strcpy(psoutput,names1[0]);
      strcat(psoutput,"_");
      strcat(psoutput,names2[0]);
      strcat(psoutput,"_");
      sprintf(str,"%d",tbegin);
      strcat(psoutput,str);
      strcat(psoutput,"_");
      sprintf(str,"%d",tend);
      strcat(psoutput,str);
      strcat(psoutput,"_");
      sprintf(str,"%d",qbegin);
      strcat(psoutput,str);
      strcat(psoutput,"_");
      sprintf(str,"%d",qend);
      strcat(psoutput,str);
      strcat(psoutput,".ps");
      int length_name_100 = strlen(psoutput);
      s=0;
      while(psoutput[s] != '\0'){
	if(psoutput[s]=='\\' || psoutput[s]=='/'){
	  psoutput[s]='_';
	}
	s++;
      }
      printf("structure %s x-y %d %d %d %d",structure,tbegin,tend,qbegin,qend);
      aliPS_color_aln(structure, psoutput, (const char**) target, (const char**) names1);
      free(structure);
      free(result);
      free(psoutput);
      for (i=0; i<n_seq+1; i++) {
	free(target[i]);
      }
      free(target);
    }
  }while(1);
  for (i=0; AS1[i]; i++) {
    free(AS1[i]); 
    free(AS2[i]); 
    free(names1[i]);
    free(names2[i]);
  }
  fclose(mrna);
  fclose(query);
}

/*
All characters not being included into ATCGUN are considered as gap characters.
 */
static int get_sequence_length_from_alignment(char *sequence){
  int counter=0;
  while(!(*sequence == '\0')){
    if(*sequence == 'A' || *sequence== 'T' || *sequence == 'C' || *sequence == 'G' || *sequence == 'U' || *sequence == 'N'){
      counter++;
    }
    sequence++;
  }
  return counter;
}

static int linear_fit(int *il_a, int *il_b, int *b_a, int *b_b){ /*get fit parameters*/
  paramT *P = NULL;
  if ((!P) || (fabs(P->temperature - temperature)>1e-6)) {
    update_fold_params();  P = scale_parameters();
    make_pair_matrix();
  }
  int internal_loop_x[13] = {6,8,10,12,14,16,18,20,22,24,26,28,30};
  int internal_loop_y[13];
  int i;
  for(i=0; i<13; i++){ //initialize the y-value for internal loops
    internal_loop_y[i] =  P->internal_loop[internal_loop_x[i]];
  }
  int sumx,sumy,sumxx,sumxy;
  sumx=sumy=sumxx=sumxy=0;
  double til_a, til_b;
  int n=8; //only take data points till int_loop size <=20; 
           //no measurement exists for larger loop && 
           //such large loop are not expected in RNA-RNA interactions.
  for(i=0; i<n; i++){
    sumx+=internal_loop_x[i];
    sumy+=internal_loop_y[i];
    sumxx+=internal_loop_x[i]*internal_loop_x[i];
    sumxy+=internal_loop_x[i]*internal_loop_y[i];
  }
  if((sumxx - (sumx*sumx)/n) < 1e-6){
    printf("divisor for internal loop is too small %d\n",(sumxx - (sumx*sumx)/n));
    error("Problem in fitting");
  }
  til_a = (sumxy - (sumx * sumy)/n)/(sumxx - (sumx*sumx)/n);
  til_b = sumy/n - (til_a * sumx/n);
  *il_a = (int) til_a;
  *il_b = (int) til_b;
  
  //###bulge####

  n=5;
  int bulge_loop_x[5] = {2,3,4,5,6};
  int bulge_loop_y[5];
  for(i=0; i<n; i++){
    bulge_loop_y[i]    =  P->bulge[bulge_loop_x[i]];
  }
  sumx=sumy=sumxx=sumxy=0;
  double tb_a, tb_b;
  for(i=0; i<n; i++){
    sumx+=bulge_loop_x[i];
    sumy+=bulge_loop_y[i];
    sumxx+=bulge_loop_x[i]*bulge_loop_x[i];
    sumxy+=bulge_loop_x[i]*bulge_loop_y[i];
  }
  tb_a = (sumxy - (sumx * sumy)/n)/(sumxx - (sumx*sumx)/n);
  tb_b = sumy/n - (tb_a * sumx/n);
  *b_a = (int) tb_a;
  *b_b = (int) tb_b;
  if((sumxx - (sumx*sumx)/n) < 1e-6){
    printf("divisor for bulge loop is too small %d\n",(sumxx - (sumx*sumx)/n));
    error("Problem in fitting");
  }
}
